# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import dataclasses
import json  # type: ignore
import logging
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1, rest_helpers, rest_streaming
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import json_format
from requests import __version__ as requests_version

from google.cloud.essential_contacts_v1.types import service

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .rest_base import _BaseEssentialContactsServiceRestTransport

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = logging.getLogger(__name__)

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=f"requests@{requests_version}",
)


class EssentialContactsServiceRestInterceptor:
    """Interceptor for EssentialContactsService.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the EssentialContactsServiceRestTransport.

    .. code-block:: python
        class MyCustomEssentialContactsServiceInterceptor(EssentialContactsServiceRestInterceptor):
            def pre_compute_contacts(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_compute_contacts(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_create_contact(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_contact(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_delete_contact(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def pre_get_contact(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_contact(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_contacts(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_contacts(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_send_test_message(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def pre_update_contact(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_update_contact(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = EssentialContactsServiceRestTransport(interceptor=MyCustomEssentialContactsServiceInterceptor())
        client = EssentialContactsServiceClient(transport=transport)


    """

    def pre_compute_contacts(
        self,
        request: service.ComputeContactsRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[service.ComputeContactsRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for compute_contacts

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EssentialContactsService server.
        """
        return request, metadata

    def post_compute_contacts(
        self, response: service.ComputeContactsResponse
    ) -> service.ComputeContactsResponse:
        """Post-rpc interceptor for compute_contacts

        Override in a subclass to manipulate the response
        after it is returned by the EssentialContactsService server but before
        it is returned to user code.
        """
        return response

    def pre_create_contact(
        self,
        request: service.CreateContactRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[service.CreateContactRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for create_contact

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EssentialContactsService server.
        """
        return request, metadata

    def post_create_contact(self, response: service.Contact) -> service.Contact:
        """Post-rpc interceptor for create_contact

        Override in a subclass to manipulate the response
        after it is returned by the EssentialContactsService server but before
        it is returned to user code.
        """
        return response

    def pre_delete_contact(
        self,
        request: service.DeleteContactRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[service.DeleteContactRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for delete_contact

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EssentialContactsService server.
        """
        return request, metadata

    def pre_get_contact(
        self,
        request: service.GetContactRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[service.GetContactRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for get_contact

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EssentialContactsService server.
        """
        return request, metadata

    def post_get_contact(self, response: service.Contact) -> service.Contact:
        """Post-rpc interceptor for get_contact

        Override in a subclass to manipulate the response
        after it is returned by the EssentialContactsService server but before
        it is returned to user code.
        """
        return response

    def pre_list_contacts(
        self,
        request: service.ListContactsRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[service.ListContactsRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for list_contacts

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EssentialContactsService server.
        """
        return request, metadata

    def post_list_contacts(
        self, response: service.ListContactsResponse
    ) -> service.ListContactsResponse:
        """Post-rpc interceptor for list_contacts

        Override in a subclass to manipulate the response
        after it is returned by the EssentialContactsService server but before
        it is returned to user code.
        """
        return response

    def pre_send_test_message(
        self,
        request: service.SendTestMessageRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[service.SendTestMessageRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for send_test_message

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EssentialContactsService server.
        """
        return request, metadata

    def pre_update_contact(
        self,
        request: service.UpdateContactRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[service.UpdateContactRequest, Sequence[Tuple[str, Union[str, bytes]]]]:
        """Pre-rpc interceptor for update_contact

        Override in a subclass to manipulate the request or metadata
        before they are sent to the EssentialContactsService server.
        """
        return request, metadata

    def post_update_contact(self, response: service.Contact) -> service.Contact:
        """Post-rpc interceptor for update_contact

        Override in a subclass to manipulate the response
        after it is returned by the EssentialContactsService server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class EssentialContactsServiceRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: EssentialContactsServiceRestInterceptor


class EssentialContactsServiceRestTransport(_BaseEssentialContactsServiceRestTransport):
    """REST backend synchronous transport for EssentialContactsService.

    Manages contacts for important Google Cloud notifications.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1
    """

    def __init__(
        self,
        *,
        host: str = "essentialcontacts.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[EssentialContactsServiceRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'essentialcontacts.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            url_scheme=url_scheme,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or EssentialContactsServiceRestInterceptor()
        self._prep_wrapped_messages(client_info)

    class _ComputeContacts(
        _BaseEssentialContactsServiceRestTransport._BaseComputeContacts,
        EssentialContactsServiceRestStub,
    ):
        def __hash__(self):
            return hash("EssentialContactsServiceRestTransport.ComputeContacts")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: service.ComputeContactsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> service.ComputeContactsResponse:
            r"""Call the compute contacts method over HTTP.

            Args:
                request (~.service.ComputeContactsRequest):
                    The request object. Request message for the
                ComputeContacts method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.service.ComputeContactsResponse:
                    Response message for the
                ComputeContacts method.

            """

            http_options = (
                _BaseEssentialContactsServiceRestTransport._BaseComputeContacts._get_http_options()
            )

            request, metadata = self._interceptor.pre_compute_contacts(
                request, metadata
            )
            transcoded_request = _BaseEssentialContactsServiceRestTransport._BaseComputeContacts._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEssentialContactsServiceRestTransport._BaseComputeContacts._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.ComputeContacts",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "ComputeContacts",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = (
                EssentialContactsServiceRestTransport._ComputeContacts._get_response(
                    self._host,
                    metadata,
                    query_params,
                    self._session,
                    timeout,
                    transcoded_request,
                )
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = service.ComputeContactsResponse()
            pb_resp = service.ComputeContactsResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_compute_contacts(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = service.ComputeContactsResponse.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.compute_contacts",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "ComputeContacts",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _CreateContact(
        _BaseEssentialContactsServiceRestTransport._BaseCreateContact,
        EssentialContactsServiceRestStub,
    ):
        def __hash__(self):
            return hash("EssentialContactsServiceRestTransport.CreateContact")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: service.CreateContactRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> service.Contact:
            r"""Call the create contact method over HTTP.

            Args:
                request (~.service.CreateContactRequest):
                    The request object. Request message for the CreateContact
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.service.Contact:
                    A contact that will receive
                notifications from Google Cloud.

            """

            http_options = (
                _BaseEssentialContactsServiceRestTransport._BaseCreateContact._get_http_options()
            )

            request, metadata = self._interceptor.pre_create_contact(request, metadata)
            transcoded_request = _BaseEssentialContactsServiceRestTransport._BaseCreateContact._get_transcoded_request(
                http_options, request
            )

            body = _BaseEssentialContactsServiceRestTransport._BaseCreateContact._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEssentialContactsServiceRestTransport._BaseCreateContact._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.CreateContact",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "CreateContact",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = (
                EssentialContactsServiceRestTransport._CreateContact._get_response(
                    self._host,
                    metadata,
                    query_params,
                    self._session,
                    timeout,
                    transcoded_request,
                    body,
                )
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = service.Contact()
            pb_resp = service.Contact.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_create_contact(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = service.Contact.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.create_contact",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "CreateContact",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _DeleteContact(
        _BaseEssentialContactsServiceRestTransport._BaseDeleteContact,
        EssentialContactsServiceRestStub,
    ):
        def __hash__(self):
            return hash("EssentialContactsServiceRestTransport.DeleteContact")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: service.DeleteContactRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ):
            r"""Call the delete contact method over HTTP.

            Args:
                request (~.service.DeleteContactRequest):
                    The request object. Request message for the DeleteContact
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.
            """

            http_options = (
                _BaseEssentialContactsServiceRestTransport._BaseDeleteContact._get_http_options()
            )

            request, metadata = self._interceptor.pre_delete_contact(request, metadata)
            transcoded_request = _BaseEssentialContactsServiceRestTransport._BaseDeleteContact._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEssentialContactsServiceRestTransport._BaseDeleteContact._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.DeleteContact",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "DeleteContact",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = (
                EssentialContactsServiceRestTransport._DeleteContact._get_response(
                    self._host,
                    metadata,
                    query_params,
                    self._session,
                    timeout,
                    transcoded_request,
                )
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

    class _GetContact(
        _BaseEssentialContactsServiceRestTransport._BaseGetContact,
        EssentialContactsServiceRestStub,
    ):
        def __hash__(self):
            return hash("EssentialContactsServiceRestTransport.GetContact")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: service.GetContactRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> service.Contact:
            r"""Call the get contact method over HTTP.

            Args:
                request (~.service.GetContactRequest):
                    The request object. Request message for the GetContact
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.service.Contact:
                    A contact that will receive
                notifications from Google Cloud.

            """

            http_options = (
                _BaseEssentialContactsServiceRestTransport._BaseGetContact._get_http_options()
            )

            request, metadata = self._interceptor.pre_get_contact(request, metadata)
            transcoded_request = _BaseEssentialContactsServiceRestTransport._BaseGetContact._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEssentialContactsServiceRestTransport._BaseGetContact._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.GetContact",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "GetContact",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = EssentialContactsServiceRestTransport._GetContact._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = service.Contact()
            pb_resp = service.Contact.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_get_contact(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = service.Contact.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.get_contact",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "GetContact",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _ListContacts(
        _BaseEssentialContactsServiceRestTransport._BaseListContacts,
        EssentialContactsServiceRestStub,
    ):
        def __hash__(self):
            return hash("EssentialContactsServiceRestTransport.ListContacts")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: service.ListContactsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> service.ListContactsResponse:
            r"""Call the list contacts method over HTTP.

            Args:
                request (~.service.ListContactsRequest):
                    The request object. Request message for the ListContacts
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.service.ListContactsResponse:
                    Response message for the ListContacts
                method.

            """

            http_options = (
                _BaseEssentialContactsServiceRestTransport._BaseListContacts._get_http_options()
            )

            request, metadata = self._interceptor.pre_list_contacts(request, metadata)
            transcoded_request = _BaseEssentialContactsServiceRestTransport._BaseListContacts._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseEssentialContactsServiceRestTransport._BaseListContacts._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.ListContacts",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "ListContacts",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = (
                EssentialContactsServiceRestTransport._ListContacts._get_response(
                    self._host,
                    metadata,
                    query_params,
                    self._session,
                    timeout,
                    transcoded_request,
                )
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = service.ListContactsResponse()
            pb_resp = service.ListContactsResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_list_contacts(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = service.ListContactsResponse.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.list_contacts",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "ListContacts",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _SendTestMessage(
        _BaseEssentialContactsServiceRestTransport._BaseSendTestMessage,
        EssentialContactsServiceRestStub,
    ):
        def __hash__(self):
            return hash("EssentialContactsServiceRestTransport.SendTestMessage")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: service.SendTestMessageRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ):
            r"""Call the send test message method over HTTP.

            Args:
                request (~.service.SendTestMessageRequest):
                    The request object. Request message for the
                SendTestMessage method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.
            """

            http_options = (
                _BaseEssentialContactsServiceRestTransport._BaseSendTestMessage._get_http_options()
            )

            request, metadata = self._interceptor.pre_send_test_message(
                request, metadata
            )
            transcoded_request = _BaseEssentialContactsServiceRestTransport._BaseSendTestMessage._get_transcoded_request(
                http_options, request
            )

            body = _BaseEssentialContactsServiceRestTransport._BaseSendTestMessage._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEssentialContactsServiceRestTransport._BaseSendTestMessage._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.SendTestMessage",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "SendTestMessage",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = (
                EssentialContactsServiceRestTransport._SendTestMessage._get_response(
                    self._host,
                    metadata,
                    query_params,
                    self._session,
                    timeout,
                    transcoded_request,
                    body,
                )
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

    class _UpdateContact(
        _BaseEssentialContactsServiceRestTransport._BaseUpdateContact,
        EssentialContactsServiceRestStub,
    ):
        def __hash__(self):
            return hash("EssentialContactsServiceRestTransport.UpdateContact")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )
            return response

        def __call__(
            self,
            request: service.UpdateContactRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> service.Contact:
            r"""Call the update contact method over HTTP.

            Args:
                request (~.service.UpdateContactRequest):
                    The request object. Request message for the UpdateContact
                method.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.service.Contact:
                    A contact that will receive
                notifications from Google Cloud.

            """

            http_options = (
                _BaseEssentialContactsServiceRestTransport._BaseUpdateContact._get_http_options()
            )

            request, metadata = self._interceptor.pre_update_contact(request, metadata)
            transcoded_request = _BaseEssentialContactsServiceRestTransport._BaseUpdateContact._get_transcoded_request(
                http_options, request
            )

            body = _BaseEssentialContactsServiceRestTransport._BaseUpdateContact._get_request_body_json(
                transcoded_request
            )

            # Jsonify the query params
            query_params = _BaseEssentialContactsServiceRestTransport._BaseUpdateContact._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.UpdateContact",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "UpdateContact",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = (
                EssentialContactsServiceRestTransport._UpdateContact._get_response(
                    self._host,
                    metadata,
                    query_params,
                    self._session,
                    timeout,
                    transcoded_request,
                    body,
                )
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = service.Contact()
            pb_resp = service.Contact.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_update_contact(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = service.Contact.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.cloud.essentialcontacts_v1.EssentialContactsServiceClient.update_contact",
                    extra={
                        "serviceName": "google.cloud.essentialcontacts.v1.EssentialContactsService",
                        "rpcName": "UpdateContact",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    @property
    def compute_contacts(
        self,
    ) -> Callable[[service.ComputeContactsRequest], service.ComputeContactsResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ComputeContacts(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def create_contact(
        self,
    ) -> Callable[[service.CreateContactRequest], service.Contact]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreateContact(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def delete_contact(
        self,
    ) -> Callable[[service.DeleteContactRequest], empty_pb2.Empty]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._DeleteContact(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_contact(self) -> Callable[[service.GetContactRequest], service.Contact]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetContact(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def list_contacts(
        self,
    ) -> Callable[[service.ListContactsRequest], service.ListContactsResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListContacts(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def send_test_message(
        self,
    ) -> Callable[[service.SendTestMessageRequest], empty_pb2.Empty]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._SendTestMessage(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def update_contact(
        self,
    ) -> Callable[[service.UpdateContactRequest], service.Contact]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._UpdateContact(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("EssentialContactsServiceRestTransport",)
